@@ -27,7 +27,7 @@
 
 from .basic import isNumber,isInteger,isComplex,Number,product
 from . import polynomials as pl
-from .trigonometric import arccos
+from .trigonometric import acos
 from .powers import sqrt
 from .num_theory import complex_polar
 import re
@@ -65,6 +65,9 @@ class Vector:
         s2 = "\n{} x {} Vector array\n".format(self.rows,self.collumns)
         return s2
 
+    def __repr__(self):
+        return self.__str__()
+
     def getMatrix(self):
         return self.matrix
 
@@ -414,6 +417,9 @@ class Matrix:
             j+=1
         return f'\n{self.rows} x {self.collumns} Matrix\n'
 
+    def __repr__(self):
+        return self.__str__()
+
     def __round__(self,ndigits : int = 1) -> "Matrix":
         """Method for rounding a Matrix"""
         __tmp__ : list = [[] for item in self.rawMatrix()]
@@ -1204,7 +1210,7 @@ def AngleBetweenVectors(vector_0 : Vector,vector_1 : Vector, degrees  : bool = F
     a : Vector = vector_0
     b : Vector = vector_1
     div : float = a.dot(b) / (magnitude(a) * magnitude(b))
-    return arccos(div,degrees=degrees)
+    return acos(div,degrees=degrees)
 
 def randomMatrix(size : tuple) -> Matrix:
     """Generates a random matrix
